/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	VillagerPlans
**
** DESCRIPTION:	Villager plans
**
**
\******************************************************************************/

#ifndef VILLAGER_PLANS_H
#define VILLAGER_PLANS_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "Behavior.h"
#include "AnimManager.h"
#include "BodyPositions.h"
#include "BehaviorTypes.h"
#include "../Villagers/Behavior.h"
#include "SkillTypes.h"

#include "../Collectables/Collectable.h"
#include "../Effects/FloatingAnim.h"
#include "../Game/Isola.h"
#include "../Puzzles/Puzzle.h"
#include "../Sound/EnvironmentalSound.h"
#include "../Utils/theStringManager.h"
#include "../World/ContentMap.h"
#include "../World/Waypoint.h"
#include "../World/Environment.h"

class CVillager;

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int VillagerPlans_c_iMaxActionPlans = 80;
const int VillagerPlans_c_iMaxBehaviorDataSize = 32;

/******************************************************************************\
** TYPE DEFINITIONS
\******************************************************************************/
enum EAction
{
	eAction_DoNothing = 0,

	eAction_Wander,
	eAction_FollowVillager,
	eAction_Wait,
	eAction_Go,
	eAction_GoRelative,
	eAction_ChangeLabel,
	eAction_FollowPath,
	eAction_Swim,
	eAction_Dance,
	eAction_Learn,
	eAction_Work,
	eAction_Climb,
	eAction_Dive,
	eAction_Jump,
	eAction_Bend,
	eAction_Embrace,
	eAction_Hide,
	eAction_Carry,
	eAction_Drop,
	eAction_AdvanceState,
	eAction_SpawnAnim,
	eAction_SpawnAnimAtVillager,
	eAction_SpawnRare,
	eAction_ShakeHead,
	eAction_Heal,
	eAction_Evangelize,
	eAction_SoundOn,
	eAction_SoundOff,
	eAction_TakeCorpse,
	eAction_DealerSay,
	eAction_AdvancePuzzle,
	eAction_ResetPuzzle,
	eAction_RefreshPuzzle,
	eAction_CheckPuzzleAdvance,
	eAction_Cheer,
	eAction_Twirl_CW,
	eAction_Twirl_CCW,
	eAction_Disco,
	eAction_DoTheMonkey,
	eAction_Revel,
	eAction_LieDown,
	eAction_Scrub,
	eAction_Exorcist,
	eAction_Joy_Twirl_CW,
	eAction_PlayAnim,
	eAction_Pushups,
	eAction_ActivateProp,
	eAction_DeactivateProp,
	eAction_Meditate,

	eAction_NewBehavior,
	eAction_NewBehaviorWithParam,

	eAction_NumberOf
};

enum EPriority
{
	ePriority_Normal = 0,
	ePriority_High,

	ePriority_NumberOf
};

enum ESpeed
{
	eSpeed_Warp = 0,
	eSpeed_Crawl = 40,
	eSpeed_Walk = 100,
	eSpeed_Run = 150,
	eSpeed_Sprint = 300
};


/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CVillagerPlans
{
public:
	
	//class structures
	struct SAdvancePuzzleData
	{
		EPuzzle				m_ePuzzle;
		int					m_iRef;
	};
	
	struct SAdvanceStateData
	{
		EWorldState			m_eState;
	};
	
	struct SFloatingAnimData
	{
		EFloatingAnim		m_eAnim;
		int					m_iXPos;
		int					m_iYPos;
	};

	struct SCarryData
	{
		ECarrying			m_eCarrying;
	};

	struct SClimbData
	{
		int					m_iHeight;
	};
	
	struct SDiveData
	{
		EAnim				m_eAnim;
		int					m_iHeight;
		EDirection			m_eDirection;
		int					m_iLandingOffset;
		float				m_fLaunchVelocity;
	};

	struct SFaceDirectionData
	{
		EDirection			m_eDirection;
		EHeadDirection		m_eHeadDirection;
	};
	
	struct SGoingData
	{
		ESpeed				m_eSpeed;
		int					m_iTargetX;
		int					m_iTargetY;
		int					m_iCounter;
	};

	struct SJumpData
	{
		int					m_iHeight;
		float				m_fLaunchVelocity;
	};
	
	struct SLearnData
	{
		ESkill				m_eSkill;
	};
	
	struct SNewBehaviorData
	{
		EBehavior			m_eBehavior;
		unsigned char		m_uiData[VillagerPlans_c_iMaxBehaviorDataSize];
		//char *				m_LabelData;
		char				m_LabelData[40 - 1];   // this MUST be the constant from villager.h, but I cant reach it from here!DEBUG  -WKH
	};

	struct SPlayAnimData
	{
		EAnim				m_eAnim;
		bool				m_bIsReversed;
		float				m_fSpeed;
	};

	struct SSayData
	{
		StringId			m_eString;
	};
	
	struct SSoundOnData
	{
		ESound				m_eSound;
		float				m_fVolume;
		ESoundType			m_eSoundType;
	};
	
	struct SSwimData
	{
		bool				m_bOnBack;
	};

	struct STakeCorpseData
	{
		int					m_iCorpse;
	};
	
	struct SWaitData
	{
		EBodyPosition		m_eBodyPosition;
		EDirection			m_eDirection;
		EHeadDirection		m_eHeadDirection;
	};
	
	struct SWanderingData
	{
		ESpeed				m_eSpeed;
		int					m_iTargetX;
		int					m_iTargetY;
	};

	struct SFollowVillagerData
	{
		ESpeed				m_eSpeed;
		int					m_iTargetX;
		int					m_iTargetY;
		int					m_iCounter;
		int					m_iVillagerIndex;
	};

	union UData
	{
		SAdvancePuzzleData	m_sAdvancePuzzleData;
		SAdvanceStateData	m_sAdvanceStateData;
		SFloatingAnimData	m_sFloatingAnimData;
		SCarryData			m_sCarryData;
		SClimbData			m_sClimbData;
		SDiveData			m_sDiveData;
		SFaceDirectionData	m_sFaceDirectionData;
		SGoingData			m_sGoingData;
		SJumpData			m_sJumpData;
		SLearnData			m_sLearnData;
		SNewBehaviorData	m_sNewBehaviorData;
		SPlayAnimData		m_sPlayAnimData;
		SSayData			m_sSayData;
		SSoundOnData		m_sSoundOnData;
		SSwimData			m_sSwimData;
		STakeCorpseData		m_sTakeCorpseData;
		SWaitData			m_sWaitData;
		SWanderingData		m_sWanderingData;
		SFollowVillagerData  m_sFollowVillagerData;
	};
	
	struct SActionPlan
	{
		//structure data
		EAction				m_eAction;
		UData				m_uData;
		int					m_misc;
		unsigned int		m_uiTimer;

		//structure methods
		//setup
		public:		SActionPlan() 
					{ 
						m_eAction = eAction_DoNothing; 
						m_uiTimer = 0; 
					}
	};
	
	//class data
	SActionPlan				m_sActionPlan[VillagerPlans_c_iMaxActionPlans];

	//class methods

	CVillagerPlans(CVillager *baseVillager) : m_villager(*baseVillager) {}

	//process
	void AdvanceCurrentPlan(/* CVillager &Villager */);
	void NextPlan(/*CVillager &Villager,*/ bool startNextPlan = true);

	void ForgetPlans(/*CVillager &Villager */);
	void StartNewBehavior(/*CVillager &Villager */);
	void ProcessCurrentPlan(/*CVillager &Villager */);

	void PlanToWander(const int iTime, const ESpeed eSpeed);
	void PlanToFollowVillager(int villagerIndex,const ESpeed eSpeed = eSpeed_Walk, const EPriority ePriority = ePriority_Normal);
	void PlanToChangeBehaviorString(char *newString);
	void PlanToChangeBehaviorString(StringId stringId);
	void PlanToWait(const int iTime, const EBodyPosition eBodyPosition, const EHeadDirection eHeadDirection);
	void PlanToWait(const int iTime, const EBodyPosition eBodyPosition, const EDirection eDirection, const EHeadDirection eHeadDirection);	
	void PlanToWait(const int iTime, const EBodyPosition eBodyPosition = eBodyPosition_Upright);

	void PlanToMeditate(const int iTime);

	void PlanToGo(const CContentMap::EObject eObject, const ESpeed eSpeed = eSpeed_Walk, const EPriority ePriority = ePriority_Normal);
	void PlanToGo(const EWaypoint eWaypoint, const ESpeed eSpeed = eSpeed_Walk, const EPriority ePriority = ePriority_Normal) ;
	void PlanToGo(const EWaypoint eWaypoint, const ldwPoint sOffset, const ESpeed eSpeed = eSpeed_Walk, const EPriority ePriority = ePriority_Normal);
	void PlanToGo(const ldwPoint sPos, const ESpeed eSpeed = eSpeed_Walk, const EPriority ePriority = ePriority_Normal);
	void PlanToGo(const ldwPoint sPos, int xTolerance, int yTolerance,const ESpeed eSpeed = eSpeed_Walk, const EPriority ePriority = ePriority_Normal);
	void PlanToGo(const int iTargetX, const int iTargetY, const ESpeed eSpeed = eSpeed_Walk, const EPriority ePriority = ePriority_Normal);
	void PlanToGo(const int iTargetX, const int iTargetY, int xTolerance, int yTolerance, const ESpeed eSpeed = eSpeed_Walk, const EPriority ePriority = ePriority_Normal);
	void PlanToGoRelative(ldwPoint sPos, const ESpeed eSpeed); 
	void PlanToSwim(const int iTime, const bool bOnBack);
	void PlanToDance(const int iTime);
	void PlanToLearn(const ESkill eSkill);
	void PlanToWork(const int iTime);
	void PlanToClimb(const int iHeight);
	void PlanToDive(const EAnim eAnim, const int iHeight, const EDirection eDirection, const int iLandingOffset);
	void PlanToJump(const int iHeight = -1);
	void PlanToBend(const int iTime);
	void PlanToCheer(const int iTime);
	void PlanToTwirlCW(const int iTime); 
	void PlanToTwirlCCW(const int iTime);
	void PlanToExorcist(const int iTime);
	void PlanToJoyTwirlCW(const int iTime);
	void PlanToDisco(const int iTime);
	void PlanToDoTheMonkey(const int iTime);
	void PlanToRevel(const int iTime);
	void PlanToLieDown(const int iTime);
	void PlanToScrub(const int iTime);
	void PlanToEmbrace();
	void PlanToHide(const int iTime);
	void PlanToCarry(const ECarrying eCarrying);
	void PlanToDrop();
	void PlanToAdvanceState(const EWorldState eWorldState);
	void PlanToAdvancePuzzle(const EPuzzle ePuzzle, const int iRef = -1);
	void PlanToResetPuzzle(const EPuzzle ePuzzle); 
	void PlanToRefreshPuzzle(const EPuzzle ePuzzle); 
	void PlanToCheckPuzzleAdvance(EPuzzle puzzle,int stage);

	void PlanToShakeHead(const int iTime, const EBodyPosition eBodyPosition);
	void PlanToHeal(const EPriority ePriority = ePriority_Normal);
	void PlanToEvangelize(const EPriority ePriority = ePriority_Normal);
	void PlanToPlaySound(const ESound eSound, const float fVolume = 1.0f, const ESoundType eSoundType = eSoundType_OneShot);
	void PlanToStopSound();
	void PlanToTakeCorpse(const int iCorpse);
	void PlanToSay(const StringId eStringId);
	void PlanToFollowPath(const int iTargetX, const int iTargetY, const ESpeed eSpeed);
	void PlanToPlayAnim(const int iTime, const EAnim eAnim, const bool bIsReversed = false, const float fSpeed = 0.0f);	
	 void PlanToPlayAnim(const int iTime, const char *animName, const bool bIsReversed = false, const float fSpeed = 0.0f);
	void PlanNewBehaviorWithParam(const EBehavior eBehavior, const SBehaviorData &sData = SBehaviorData());
	void PlanNewBehavior(const EBehavior eBehavior);
	void PlanToDoPushups(const int iTime);
	void PlanToSpawnAnim(const EFloatingAnim eAnim, int xPos, int yPos);
	void PlanToSpawnAnimAtVillager(const EFloatingAnim eAnim, int xPos, int yPos);
	void PlanToSpawnRareCollectible();
	void PlanToActivateProp(EPropEnum eWhichProp);
	void PlanToDeactivateProp(EPropEnum eWhichProp);


	//access
	const bool IsCarrying() const { return m_sActionPlan[0].m_eAction == eAction_Carry; }
	const bool IsGoing() const { return m_sActionPlan[0].m_eAction == eAction_Go; }
	const bool IsIdling() const { return m_sActionPlan[0].m_eAction == eAction_DoNothing; }
	const bool IsWaiting() const { return m_sActionPlan[0].m_eAction == eAction_Wait; }
	const bool HasCurrentPlanExpired() const;
	const EAction CurrentAction() const  { return m_sActionPlan[0].m_eAction; }
	const EAction NextAction() const  { return m_sActionPlan[1].m_eAction; }

private:

	void AddPlan(const SActionPlan sPlan, const EPriority ePriority);

	CVillager& m_villager;
};

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/